<?php
namespace Ep\Cache;

use Ep\Cache\Exception\OptionRequiredException;
use Interop\Container\ContainerInterface;
use Psr\Cache\CacheItemPoolInterface;
use Zend\ServiceManager\AbstractPluginManager;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Factory\AbstractFactoryInterface;
use Zend\ServiceManager\PluginManagerInterface;

/**
 * Stvara novu instancu CacheAdaptera zadanu u konfiguraciji.
 *
 * @package Ep\Cache
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AbstractCacheFactory implements AbstractFactoryInterface
{
    const CONFIG_KEY = 'caches';

    /** @var array */
    private $config;
    /** @var PluginManagerInterface */
    private $adapters;

    /**
     * Stvara adapter iz konfiguracije.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return CacheItemPoolInterface
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $adapters = $this->getAdapterPluginManager($container);
        $config = $options ?? $this->getConfig($container)[$requestedName] ?? [];

        if (false === $config || empty($config)) {
            throw new ServiceNotCreatedException(sprintf(
                "Cache s imenom '%s' ne postoji u konfiguraciji.", $requestedName
            ));
        }

        if (!isset($config['adapter']) || empty($config['adapter'])) {
            throw new OptionRequiredException("Konfiguracija mora sadržavati polje 'adapter'.");
        }

        return $adapters->get($config['adapter'], $config['options'] ?? []);
    }

    /**
     * Da li imamo konfiguraciju za dato ime.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        if ('config' === $requestedName) {
            return false;
        }

        $config = $this->getConfig($container);

        if (empty($config)) {
            return false;
        }

        return isset($config[$requestedName]) && is_array($config[$requestedName]) && !empty($config[$requestedName]);
    }

    /**
     * Vraća plugin manager za cache adaptere.
     *
     * @param ContainerInterface $container
     *
     * @return PluginManagerInterface|AbstractPluginManager
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    private function getAdapterPluginManager(ContainerInterface $container): PluginManagerInterface
    {
        if (null === $this->adapters) {
            $this->adapters = $container->has(Adapter\PluginManager::class)
                ? $container->get(Adapter\PluginManager::class)
                : new Adapter\PluginManager($container);
        }

        return $this->adapters;
    }

    /**
     * Izvlači konfiguracije cacheva iz datog containera.
     *
     * @param ContainerInterface $container
     *
     * @return array
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    private function getConfig(ContainerInterface $container)
    {
        if (null === $this->config) {
            $this->config = $container->get('config')[self::CONFIG_KEY] ?? [];
        }

        return $this->config;
    }
}