<?php
namespace Ep\Http\Validator;

use Psr\Http\Message\ResponseInterface;
use Zend\Validator\AbstractValidator;
use Zend\Validator\NotEmpty;
use Zend\Validator\ValidatorInterface;

/**
 * Validira vrijednost headera odgovora.
 *
 * @package Ep\Http\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class HeaderValue extends AbstractValidator
{
    const NO_HEADER            = 'http_header_value.no_header';
    const INVALID_HEADER       = 'http_header_value.invalid';
    const INVALID_HEADER_VALUE = 'http_header_value.invalid_value';

    /** @var array */
    protected $messageTemplates = [
        self::NO_HEADER            => 'Header %header% ne postoji u odgovoru.',
        self::INVALID_HEADER       => 'Header %header% je neispravan: %message%.',
        self::INVALID_HEADER_VALUE => "Header %header% treba imati vrijednost '%header_value%', a ima '%value%'.",
    ];

    /** @var array */
    protected $messageVariables = [
        'header' => ['options' => 'header'],
        'header_value' => ['options' => 'header_value'],
        'message' => ['options' => 'message'],
    ];

    /** @var array */
    protected $options = [
        'header' => '',
        'header_value' => '',
        'message' => '',
    ];

    /** @var ValidatorInterface */
    protected $validator;

    /**
     * Vraća validator korišten za validaciju vrijednosti headera.
     *
     * @return ValidatorInterface
     */
    public function getValidator(): ValidatorInterface
    {
        if (null === $this->validator) {
            $this->validator = new NotEmpty();
        }

        return $this->validator;
    }

    /**
     * Validator korišten za validaciju vrijednosti headera.
     *
     * @param ValidatorInterface $validator
     */
    public function setValidator(ValidatorInterface $validator): void
    {
        $this->validator = $validator;
    }

    /**
     * Returns true if and only if $value meets the validation requirements
     *
     * If $value fails validation, then this method returns false, and
     * getMessages() will return an array of messages that explain why the
     * validation failed.
     *
     * @param  ResponseInterface|mixed $value
     *
     * @return bool
     */
    public function isValid($value)
    {
        if (!$value instanceof ResponseInterface) {
            throw new \InvalidArgumentException('Ovaj validator može validirati samo HTTP odgovore.');
        }

        $header = $this->options['header'];

        if (empty($header)) {
            throw new \RuntimeException('Nije postavljeno ime headera kojeg validiramo.');
        }

        if (!$value->hasHeader($header)) {
            $this->error(self::NO_HEADER);
            return false;
        }

        if (!empty($this->options['header_value'])) {
            if (!in_array($this->options['header_value'], $value->getHeader($header))) {
                $this->error(self::INVALID_HEADER_VALUE);
                return false;
            }
        } else {
            $validator = $this->getValidator();

            if (!$validator->isValid($value->getHeaderLine($header))) {
                $this->options['message'] = implode(' ', $validator->getMessages());
                $this->error(self::INVALID_HEADER);
                return false;
            }
        }

        return true;
    }
}