<?php
namespace EpTest\Http\Validator;

use Ep\Http\Validator\HeaderValue;
use Ep\Validator\MinVersion;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Zend\Diactoros\Response;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\Validator\ValidatorInterface;
use Zend\Validator\ValidatorPluginManager;

/**
 * Class HeaderValueTest
 *
 * @package Ep\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class HeaderValueTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testIsRegistered()
    {
        $container = $this->createContainer();
        /** @var PluginManagerInterface $validators */
        $validators = $container->get(ValidatorPluginManager::class);

        $this->assertTrue($validators->has('http_header_value'));
        $this->assertInstanceOf(HeaderValue::class, $validators->get('http_header_value'));
    }

    /**
     * @expectedException \InvalidArgumentException
     */
    public function testWillValidateOnlyHttpResponses()
    {
        $validator = new HeaderValue();
        $validator->isValid('string');
    }

    /**
     * @expectedException \RuntimeException
     */
    public function testWillThrowExceptionIfHeaderNotSet()
    {
        $validator = new HeaderValue();
        $validator->isValid(new Response());
    }

    public function testHasADefaultValidator()
    {
        $validator = new HeaderValue([
            'header' => 'Content-Type',
        ]);

        $this->assertInstanceOf(ValidatorInterface::class, $validator->getValidator());
        $this->assertFalse($validator->isValid(new Response\EmptyResponse()));
    }

    public function testWillFailIfNoHeader()
    {
        $validator = new HeaderValue([
            'header' => 'X-Invalid',
        ]);

        $this->assertFalse($validator->isValid(new Response\HtmlResponse('')));
        $this->assertArrayHasKey(HeaderValue::NO_HEADER, $validator->getMessages());
    }

    public function testWillFailForInvalidHeader()
    {
        $validator = new HeaderValue([
            'header' => 'X-Version',
            'validator' => new MinVersion([
                'min_version' => '1.2.3',
            ]),
        ]);

        $response = (new Response())
            ->withAddedHeader('X-Version', '1.2.2');

        $this->assertFalse($validator->isValid($response));
        $this->assertArrayHasKey(HeaderValue::INVALID_HEADER, $validator->getMessages());
    }

    public function testWillValidateValidHeaders()
    {
        $validator = new HeaderValue([
            'header' => 'X-Version',
            'validator' => new MinVersion([
                'min_version' => '1.2.3',
            ]),
        ]);

        $response = (new Response())
            ->withAddedHeader('X-Version', '1.2.3');

        $this->assertTrue($validator->isValid($response));
    }

    public function testWillValidateAgainstStringValueIfSet()
    {
        $validator = new HeaderValue([
            'header' => 'Content-Type',
            'header_value' => 'application/json',
        ]);

        $this->assertFalse($validator->isValid(new Response\HtmlResponse('')));
        $this->assertTrue($validator->isValid(new Response\JsonResponse([])));
    }

    public function testCanValidateMultiValueHeaders()
    {
        $validator = new HeaderValue([
            'header' => 'Content-Type',
            'header_value' => 'application/json',
        ]);

        $response = (new Response())
            ->withAddedHeader('Content-Type', ['text/html', 'application/json']);

        $this->assertTrue($validator->isValid($response));
    }
}